<?php

declare(strict_types=1);

namespace Drupal\captcha_captchetat\Helper;

use Drupal\captcha_captchetat\Client\CaptchaClientInterface;
use Drupal\Core\Url;

/**
 * Provide a client helper.
 */
final class ClientHelper implements ClientHelperInterface {

  /**
   * The "CaptchEtat" captcha client.
   *
   * @var \Drupal\captcha_captchetat\Client\CaptchaClientInterface
   */
  protected CaptchaClientInterface $client;

  /**
   * Constructs a "ClientHelper" object.
   *
   * @param \Drupal\captcha_captchetat\Client\CaptchaClientInterface $captcha_client
   *   The Captcha client.
   */
  public function __construct(
    CaptchaClientInterface $captcha_client
  ) {
    $this->client = $captcha_client;
  }

  /**
   * Convert relative paths to "CaptchEtat module" path… Hum… ok!
   *
   * The captcha service will generate html code with links having the URL "/api/simple-captcha-endpoint?[parameter]…".
   * It will be necessary replace these URLs with the URL of the API used to generate the captcha.
   *
   * @return string
   *   The HTML with absolute URLs.
   */
  private function captchaConvertPaths2AbsoluteUrls(string $html): string {
    return str_replace(
      '/api/simple-captcha-endpoint',
      Url::fromRoute('captcha_captchetat.object')->toString(),
      $html
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCaptcha(
    string $mode = CaptchaClientInterface::MODE_FRONTAL,
    string $objectType = CaptchaClientInterface::OBJECT_TYPE_HTML,
    ?string $captchaType = NULL,
    ?string $captchaId = NULL
  ): string {
    $html = $this->client->captcha($mode, $objectType, $captchaType, $captchaId);

    return isset($html) ? $this->captchaConvertPaths2AbsoluteUrls($html) : '';
  }

  /**
   * {@inheritdoc}
   */
  public function getVersion(): string {
    return $this->client->version() ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function isAvailable(): bool {
    $data = $this->client->healthcheck();

    return (
      isset($data['status']) &&
      $data['status'] === 'UP'
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isCaptchaValid(string $captchaId, string $code): bool {
    $data = $this->client->captchaValidate($captchaId, $code);

    return $data === 'true';
  }

}
